require 'locomotive_plugins'
require 'json'

require_relative 'booking_document'
require_relative 'helper'

module Locomotive
    module Booking
        class GetBookingsTag < ::Liquid::Tag
            ## Extend the String class with is_i? to determine whether the string is a number or not.
            class ::String
                def is_i?
                    ## The regex checks if the String begins with a number, only has numbers in the middle and ends with a number.
                    /\A\d+\Z/.match(self)
                end
            end

            RESOURCE_ID_INTERNAL_KEY = "resource_id"
            RESOURCE_ID_EXTERNAL_KEY = "resourceId"

            def initialize(tag_name, markup, tokens, context)
                @calendar_week  = nil
                @calendar_year  = nil
                @resource_id    = nil

                markup.scan(::Liquid::TagAttributes) do |key, value|
                    ## Apparently you should remove possible quotes from the value.
                    value_minus_quotes = Locomotive::Booking::Helper.strip_quotes_from_value(value)
                    Locomotive::Booking::Helper.setInstanceVariableIfExists(self, key, value_minus_quotes)
                end
                super
            end

            def render(context)
                ## Resolve the values if it's an expression rather than a
                ## constant.
                year        = Locomotive::Booking::Helper.evaluateWithValueAndContext(@calendar_year, context)
                week        = Locomotive::Booking::Helper.evaluateWithValueAndContext(@calendar_week, context)
                resourceId  = Locomotive::Booking::Helper.evaluateWithValueAndContext(@resource_id, context)

                if year.is_i? and week.is_i? and resourceId
                    year = year.to_i
                    week = week.to_i
                else
                    ::Locomotive.log("year: #{year}, week: #{week}")
                    return nil.to_json
                end

                if week < 1 or week > 52
                    nil.to_json
                else
                    ::Locomotive.log("GetBookingsTag: resourceId: #{resourceId}")
                    booking_data = BookingDocument::Availability.without(:_id).where(year: year, week: week, resource_id: resourceId).first
                    if booking_data
                        ::Locomotive.log("found booking_data")
                        
                        ## Convert the resulting hash so that the key is 'resourceId' rather than 'resource_id' to preserve the external API.
                        resource_id = booking_data[:resource_id]
                        new_booking_data_hash = booking_data.as_document.except(RESOURCE_ID_INTERNAL_KEY)

                        new_booking_data_hash[RESOURCE_ID_EXTERNAL_KEY] = resource_id
                        new_booking_data_hash.to_json
                    else
                        ::Locomotive.log("Didn't find booking_data.")
                        nil.to_json
                    end
                end
            end

            def render_disabled(context)
                nil.to_json
            end
        end
    end
end
