require 'uri'
require 'json'

module Locomotive
    module Booking
        # Class that contains different helper methods.
        class Helper
            SUCCESS_KEY = "success"

            # Removes all single and double quotes from the given +value+.
            def self.strip_quotes_from_value(value)
                value.gsub(/"|'/, '')
            end

            # Returnes a JSONObject-String with { "success": true }.
            def self.success_json
                { SUCCESS_KEY => true }.to_json
            end

            # Returnes a JSONObject-String with { "success": false }.
            def self.failure_json
                { SUCCESS_KEY => false }.to_json
            end

            # Evaluates the given value with a context.
            #
            # This either returns the evaluated value or if that failed, the
            # value as it is.
            def self.evaluateWithValueAndContext(value, context)
                new_value       = value
                evaluated_value = context[value]
                new_value       = evaluated_value unless evaluated_value.nil? or evaluated_value.empty?

                new_value
            end

            # Sets the the instance variable of an object for a variable name
            # to a value if the object has defined the instance variable.
            def self.setInstanceVariableIfExists(object, variable_name, value)
                if not variable_name.match(/^@/)
                    ## Need to prepend the '@' since it's an instance variable.
                    variable_name = "@#{variable_name}"
                end

                if object.instance_variable_defined? variable_name
                    object.instance_variable_set(variable_name, value)
                end
            end

            # Takes an escaped URI, unescapes it and parses it into a JSON.
            # Return nil if the parsing failed.
            def self.unescapeAndParseURIToJSON(escapedURI)
                begin
                    json_string = URI.unescape(escapedURI)
                    JSON.parse(json_string)
                rescue JSON::ParserError
                    ::Locomotive.log("Helper: parse error")
                    return nil
                end
            end

            # Scans over the given markup and retrieves the value for the given key.
            # Return nil if the key couldn't be found.
            def self.markupScanForKey(key, markup)
                markup.scan(::Liquid::TagAttributes) do |key, value|
                    ## Apparently you should remove possible quotes from the value.
                    value_minus_quotes = self.strip_quotes_from_value(value)

                    if key == key
                        return value_minus_quotes
                    end
                end

                return nil
            end
        end
    end
end
